package zap;

import VASSAL.build.*;
import VASSAL.build.module.*;
import VASSAL.command.*;

import javax.swing.*;
import java.awt.event.*;
import java.util.*;

/**
 * Tension index class for the VASSAL tutorial
 */
public class Tension extends AbstractConfigurable implements CommandEncoder, GameComponent {
    private int index = 0;
    private int minChange = 0;
    private int maxChange = 0;

    private Random rand = new Random();
   
    private JButton addButton; // Adds an increment to the tension counter
    private JButton showButton; // Shows the current total tension

    public void addToIndex(int change) {
	index += change;
    }
    
    public int getIndex() {
	return index;
    }

    public int newIncrement() {
	return (int)(rand.nextFloat() * (maxChange - minChange + 1))
	    + minChange;
    }
    
    private void incrementButtonPressed() {
	GameModule mod = GameModule.getGameModule();
	int change = newIncrement();
	Command c = new
	    Chatter.DisplayText(mod.getChatter(),"Tension changes by "+change);
	c.append(new Incr(this,change));
	c.execute();
	mod.sendAndLog(c);
    }

    private void totalButtonPressed() {
	GameModule.getGameModule().getChatter().show("Tension index is "+index);
    }
    
    public static final String MIN = "min";
    public static final String MAX = "max";
    public void setAttribute(String key, Object value) {
	if (MIN.equals(key)) {
	    if (value instanceof String) {
		minChange = Integer.parseInt((String)value);
	    }
	    else if (value instanceof Integer) {
		minChange = ((Integer)value).intValue();
	    }
	}
	else if (MAX.equals(key)) {
	    if (value instanceof String) {
		maxChange = Integer.parseInt((String)value);
	    }
	    else if (value instanceof Integer) {
		maxChange = ((Integer)value).intValue();
	    }
	}
    }
    public String[] getAttributeNames() {
	return new String[]{MIN,MAX};
    }
    public String[] getAttributeDescriptions() {
	return new String[]{"Minimum increment","Maximum increment"};
    }
    public Class[] getAttributeTypes() {
	return new Class[]{Integer.class,Integer.class};
    }
    public String getAttributeValueString(String key) {
	if (MIN.equals(key)) {
	    return ""+minChange;
	}
	else if (MAX.equals(key)) {
	    return ""+maxChange;
	}
	else {
	    return null;
	}
    }
    public void addTo(Buildable parent) {
	GameModule mod = (GameModule)parent;

	mod.addCommandEncoder(this);
	mod.getGameState().addGameComponent(this);

	addButton = new JButton("Increment");
	addButton.setAlignmentY(0.0F);
	addButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    incrementButtonPressed();
		}});
	mod.getToolBar().add(addButton);
	
	showButton = new JButton("Show total");
	showButton.setAlignmentY(0.0F);
	showButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt) {
		    totalButtonPressed();
		}
	    });
	mod.getToolBar().add(showButton);
    }
    public void removeFrom(Buildable parent) {
	GameModule mod = (GameModule)parent;

	mod.removeCommandEncoder(this);
	mod.getGameState().removeGameComponent(this);

	mod.getToolBar().remove(addButton);
	mod.getToolBar().remove(showButton);
    }

    public VASSAL.build.module.documentation.HelpFile getHelpFile() {
	return null;
    }

    public Class[] getAllowableConfigureComponents() {
	return new Class[0];
    }
    public void setup(boolean gameStarting) {
	if (!gameStarting) {
	    index = 0;
	}
    }

    public Command getRestoreCommand() {
	return new Incr(this,index);
    }

    public static final String COMMAND_PREFIX = "TENSION:";
    public String encode(Command c) {
	if (c instanceof Incr) {
	    return COMMAND_PREFIX+((Incr)c).getChange();
	}
	else {
	    return null;
	}
    }
    public Command decode(String s) {
	if (s.startsWith(COMMAND_PREFIX)) {
	    return new Incr(this,Integer.parseInt(s.substring(COMMAND_PREFIX.length())));
	}
	else {
	    return null;
	}
    }

    public static class Incr extends Command {
	private Tension target;
	private int change;

	public Incr(Tension target, int change) {
	    this.target = target;
	    this.change = change;
	}       
	protected void executeCommand() {
	    target.addToIndex(change);
	}
	protected Command myUndoCommand() {
	    return new Incr(target,-change);
	}	
	public int getChange() {
	    return change;
	}
    }
}
